#import pathlib
from setuptools import setup

# The directory containing this file
#HERE = pathlib.Path(__file__).parent

# The text of the README file
#README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="drukbam",
    version="0.1",
    description="plots sorted indexed bam files",
    long_description='test',
    long_description_content_type="text/markdown",
    url="https://github.com/StephanHolgerD/DrukBam",
    author="Stephan Holger Drukewitz",
    author_email="steph-druk@web.de",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["DrukBamon"],
    include_package_data=True,
    install_requires=["pandas", "matplotlib",'pysam','multiprocessing','tqdm',],
    entry_points={
        "console_scripts": [
            "DrukBam=DrukBam.__main__:main",
        ]
    },
)



import pysam
import pandas as pd
import matplotlib.pyplot as plt
from tqdm import tqdm
from multiprocessing import Pool
from DrukBam.MapPlot import PlotMapping
import sys
from time import time
class VcfPlotter():
    def __init__(self,vcf,mapping,coverage=200,flag='None',chunksize=1000,padding=20,direction=False,schematic=False,threads=1,
    fasta=None,output='current working directory',out_name='name of mapping',style='classic',outfmt='pdf',outlineoff=False):
        self.outputdir=output
        self.out_name=out_name
        self.mapping=mapping
        self.vcf=vcf
        self.padding=padding
        self.Fontsize=3
        self.threads=threads
        self.flag=flag
        self.direction=direction
        if direction:
            self.maxHeight=coverage
        else:
            self.maxHeight=coverage
        self.schematic=schematic
        self.threads=threads
        self.fasta=fasta
        self.chunksize=chunksize
        self.style=style
        self.outfmt=outfmt
        self.outlineoff=outlineoff
    def PlotV(self,c,s,e):
            ploter=PlotMapping(
                self.mapping,
                c,
                int(s),
                int(e),
                flag=self.flag,
                schematic=self.schematic,
                direction=self.direction,
                coverage=self.maxHeight,
                threads=self.threads,
                fasta=self.fasta,
                out_name=self.out_name,
                output=self.outputdir,
                chunksize=self.chunksize,
                vcf=True,
                outfmt=self.outfmt,
                style=self.style,
                outlineoff=self.outlineoff)
            ploter.Plot()
    def MultiPlot(self):
        multi=[]
        with pysam.VariantFile(self.vcf) as v:
            for record in v:
                multi.append((record.chrom,(record.pos-self.padding),(record.pos+self.padding)))
        print(multi)
        with Pool(processes=self.threads) as pool:
            results = pool.starmap(self.PlotV, multi)



import pysam
import pandas as pd
import matplotlib.pyplot as plt
from tqdm import tqdm
from multiprocessing import Pool
from DrukBam.bamCalc import CalcMapping
from DrukBam.PlotCalc import CalcPlot
import sys
class PlotMapping():
    def __init__(self,mapping,chrom,start,end,coverage=200,flag='None',direction=False,schematic=False,chunksize=1000,threads=1,fasta=None,output='current working directory',
    out_name='name of mapping',vcf=False,style='classic',outfmt='pdf',outlineoff=False):
        self.outputdir=output
        self.out_name=out_name
        self.mapping=mapping
        self.chrom=chrom
        self.start=start
        self.end=end
        self.Fontsize=3
        self.threads=threads
        self.flag=flag
        self.chunksize=chunksize
        self.direction=direction
        if direction:
            self.maxHeight=coverage
        else:
            self.maxHeight=coverage
        self.schematic=schematic
        self.threads=threads
        self.fasta=fasta
        self.style=style
        self.CalcMapping=CalcMapping(mapping,chrom,start,end,coverage=self.maxHeight,flag=self.flag, threads=self.threads,chunksize=chunksize)
        self.CalcPlot=CalcPlot(mapping,chrom,self.start,self.end,coverage=self.maxHeight,flag=self.flag, threads=self.threads,fasta=self.fasta,style=self.style,outlineoff=outlineoff)
        self.vcf=vcf
        self.outfmt=outfmt
    def savePlot(self,schmematic=False,direction=False,reference=False):
        if self.out_name=='name of mapping':
            o=self.mapping.split('/')[-1]
        else:
            o=self.out_name
        if self.outputdir=='current working directory':
            od='./'
        else:
            od=self.outputdir
        r=''
        d=''
        s=''
        if schmematic:
            r='S'
        if direction:
            d='D'
        if reference != 'None':
            r='R'

        self.fig.savefig('{}/{}_{}_{}_{}{}{}{}.{}'.format(od,o,self.chrom,str(self.start),str(self.end),s,d,r,self.outfmt),bbox_inches='tight',dpi=400)
        plt.close()

    def Plot(self):
        if self.schematic and not self.direction:
            self.PlotSchmematic()
            self.savePlot(schmematic=True)
            return
        if self.schematic and self.direction:
            self.PlotSchematicDir()
            self.savePlot(schmematic=True,direction=True)
            return
        if self.direction and not self.schematic:
            if (self.end - self.start) > self.chunksize:
                print('span larger than chunksize, please increase chunksize, calc. speed will slow down with large chunksizes')
                sys.exit()
            self.PlotNucDir()
            self.savePlot(direction=True,reference=self.fasta)
            return
        if not self.direction and not self.schematic:
            if (self.end - self.start) > self.chunksize:
                print('span larger than chunksize, please increase chunksize, calc. speed will slow down with large chunksizes')
                sys.exit()
            self.PlotNuc()
            self.savePlot(reference=self.fasta)
            return


    def CalcFRWD(self):
        span=self.end-self.start
        multi=[]
        start=self.start
        end=self.start+self.chunksize
        if span<self.chunksize:
            results=[self.CalcMapping.plotList(self.chrom,self.start,self.end,True,False)]
            multi.append((self.chrom,self.start,self.end,True,False))
        else:
            for _ in range(0,int(span/self.chunksize)):
                multi.append((self.chrom,start,end,True,False))
                start=end
                end=end+self.chunksize
            if span/self.chunksize - int(span/self.chunksize)!=0:
                multi.append((self.chrom,start,int(start+((span/self.chunksize - int(span/self.chunksize))*self.chunksize)),True,False))
            with Pool(processes=self.threads) as pool:
                results = pool.starmap(self.CalcMapping.plotList, multi)
        return results,multi

    def CalcRVRS(self):
        span=self.end-self.start
        multi=[]
        start=self.start
        end=self.start+self.chunksize
        if span<self.chunksize:
            results=[self.CalcMapping.plotList(self.chrom,self.start,self.end,False,True)]
            multi.append((self.chrom,self.start,self.end,False,True))
        else:
            for _ in range(0,int(span/self.chunksize)):
                multi.append((self.chrom,start,end,False,True))
                start=end
                end=end+self.chunksize
            if span/self.chunksize - int(span/self.chunksize)!=0:
                multi.append((self.chrom,start,int(start+((span/self.chunksize - int(span/self.chunksize))*self.chunksize)),False,True))
            with Pool(processes=self.threads) as pool:
                results = pool.starmap(self.CalcMapping.plotList, multi)
        return results,multi

    def Calc(self):
        span=self.end-self.start
        multi=[]
        start=self.start
        end=self.start+self.chunksize
        if span < self.chunksize:
            results=[self.CalcMapping.plotList(self.chrom,self.start,self.end,False,False)]
            multi.append((self.chrom,self.start,self.end,False,False))
        else:
            for _ in range(0,int(span/self.chunksize)):
                multi.append((self.chrom,start,end,False,False))
                start=end
                end=end+self.chunksize
            if span/self.chunksize - int(span/self.chunksize)!=0:
                multi.append((self.chrom,start,int(start+((span/self.chunksize - int(span/self.chunksize))*self.chunksize)),False,False))
            with Pool(processes=self.threads) as pool:
                results = pool.starmap(self.CalcMapping.plotList, multi)
        return results,multi




    def PlotSchematicDir(self):
        resultsR,multiR=self.CalcRVRS()
        resultsF,multiF=self.CalcFRWD()

        self.fig,self.ax=self.CalcPlot.startPlot(len(resultsR),self.direction,self.schematic)

        for p,chunk in enumerate(zip(resultsR,multiR)):
            direction='Reverse'
            if resultsR[p]==[] and len(resultsR)==1:
                self.CalcPlot.AxSet(self.ax[0,p],chunk[1][1],chunk[1][2],direction=direction)
                self.ax[0,p].spines['top'].set_visible(False)
                self.ax[0,p].get_xaxis().set_visible(False)
                continue

            if len(resultsR)==1:
                d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                self.CalcPlot.AxSet(self.ax[0],chunk[1][1],chunk[1][2],direction=direction)
                self.CalcPlot.plotChunk(d,self.ax[0],chunk[1][1],chunk[1][2])
                self.ax[0].spines['top'].set_visible(False)
                self.ax[0].get_xaxis().set_visible(False)
                continue

            if p==0:
                d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                self.CalcPlot.AxSet(self.ax[0,p],chunk[1][1],chunk[1][2],direction=direction)
                self.CalcPlot.plotChunk(d,self.ax[0,p],chunk[1][1],chunk[1][2])
                self.ax[0,p].get_xaxis().set_visible(False)
                self.ax[0,p].spines['top'].set_visible(False)
                self.ax[0,p].get_xaxis().set_visible(False)
                continue

            else:
                if resultsR[p-1]==[]:
                    d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                    self.CalcPlot.AxSet(self.ax[0,p],chunk[1][1],chunk[1][2],chunk=True)
                    self.CalcPlot.plotChunk(d,self.ax[0,p],chunk[1][1],chunk[1][2])
                    self.ax[0,p].spines['top'].set_visible(False)
                    self.ax[0,p].get_xaxis().set_visible(False)

                else:
                    d=self.CalcMapping.PlotreadsDF(resultsR[p-1],resultsR[p],multiR[p-1][2])
                    self.CalcPlot.AxSet(self.ax[0,p],chunk[1][1],chunk[1][2],chunk=True)
                    self.CalcPlot.plotChunk(d,self.ax[0,p],chunk[1][1],chunk[1][2])
                    self.ax[0,p].spines['top'].set_visible(False)
                    self.ax[0,p].get_xaxis().set_visible(False)

        for p,chunk in enumerate(zip(resultsF,multiF)):
            direction='Forward'


            if resultsF[p]==[] and len(resultsR)==1:
                self.CalcPlot.AxSet(self.ax[1,p],chunk[1][1],chunk[1][2],direction=direction)
                continue

            if resultsF[p]==[]:
                self.CalcPlot.AxSet(self.ax[1,p],chunk[1][1],chunk[1][2],direction=direction)


            if len(resultsF)==1:
                d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                self.CalcPlot.AxSet(self.ax[1],chunk[1][1],chunk[1][2],direction=direction)
                self.ax[1].spines['bottom'].set_visible(False)
                self.ax[1].spines['top'].set_visible(False)
                self.CalcPlot.plotChunk(d,self.ax[1],chunk[1][1],chunk[1][2])
                continue

            if p==0:
                d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                self.CalcPlot.AxSet(self.ax[1,p],chunk[1][1],chunk[1][2],direction=direction,chunk=True)
                self.ax[1,p].spines['bottom'].set_visible(False)
                self.ax[1,p].spines['top'].set_visible(False)
                self.CalcPlot.plotChunk(d,self.ax[1,p],chunk[1][1],chunk[1][2])
            else:
                if resultsF[p-1]==[]:
                    d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                    self.CalcPlot.AxSet(self.ax[1,p],chunk[1][1],chunk[1][2],chunk=True)
                    self.ax[1,p].spines['bottom'].set_visible(False)
                    self.ax[1,p].spines['top'].set_visible(False)
                    self.CalcPlot.plotChunk(d,self.ax[1,p],chunk[1][1],chunk[1][2])
                else:
                    d=self.CalcMapping.PlotreadsDF(resultsF[p-1],resultsF[p],multiF[p-1][2])
                    self.CalcPlot.AxSet(self.ax[1,p],chunk[1][1],chunk[1][2],chunk=True)
                    self.ax[1,p].spines['bottom'].set_visible(False)
                    self.ax[1,p].spines['top'].set_visible(False)
                    self.CalcPlot.plotChunk(d,self.ax[1,p],chunk[1][1],chunk[1][2])




    def PlotNucDir(self):
        resultsR,multiR=self.CalcRVRS()
        resultsF,multiF=self.CalcFRWD()
        if len(multiR)>1:
            print('nuc plots over 1000nt not supported due to fontsize')
            sys.exit()
        self.fig,self.ax=self.CalcPlot.startPlot(len(resultsR),self.direction,self.schematic)

        for p,chunk in enumerate(zip(resultsR,multiR)):
            direction='Reverse'


            if resultsR[p]==[]:
                self.CalcPlot.AxSet(self.ax[0],chunk[1][1],chunk[1][2], direction=direction,vcf=self.vcf)
                self.ax[0].get_xaxis().set_visible(False)
                self.ax[0].spines['bottom'].set_visible(False)

                continue

            if len(resultsR)==1:
                d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                self.CalcPlot.AxSet(self.ax[0],chunk[1][1],chunk[1][2], direction=direction,vcf=self.vcf)

                self.CalcPlot.PlotNucChunk(d,self.ax[0],chunk[1][1],chunk[1][2],flag=self.flag)
                self.ax[0].get_xaxis().set_visible(False)
                self.ax[0].spines['bottom'].set_visible(False)

                continue


        for p,chunk in enumerate(zip(resultsF,multiF)):
            direction='Forward'
            if resultsF[p]==[]:
                self.CalcPlot.AxSet(self.ax[1],chunk[1][1],chunk[1][2], direction=direction,vcf=self.vcf)
                continue

            if len(resultsF)==1:
                d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])

                self.ax[1].spines['bottom'].set_visible(False)
                self.CalcPlot.AxSet(self.ax[1],chunk[1][1],chunk[1][2], direction=direction,vcf=self.vcf)

                self.CalcPlot.PlotNucChunk(d,self.ax[1],chunk[1][1],chunk[1][2],flag=self.flag)

                continue

        self.CalcPlot.PlotFasta(self.ax[0])
        self.CalcPlot.PlotFasta(self.ax[1])


    def PlotNuc(self):
        results,multi=self.Calc()
        self.fig,self.ax=self.CalcPlot.startPlot(len(results),self.direction,self.schematic)
        if len(multi)>1:
            print('nuc plots over 1000nt not supported due to fontsize')
            sys.exit()
        for p,chunk in enumerate(zip(results,multi)):

            if results[p]==[]:

                self.CalcPlot.AxSet(self.ax,chunk[1][1],chunk[1][2])
                continue
            else:
                d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                self.CalcPlot.AxSet(self.ax,chunk[1][1],chunk[1][2],vcf=self.vcf)
                self.ax.spines['bottom'].set_visible(False)
                self.CalcPlot.PlotNucChunk(d,self.ax,chunk[1][1],chunk[1][2],flag=self.flag)
                continue
        self.CalcPlot.PlotFasta(self.ax)





    def PlotSchmematic(self):
        results,multi=self.Calc()
        self.fig,self.ax=self.CalcPlot.startPlot(len(results),self.direction,self.schematic)
        for p,chunk in enumerate(zip(results,multi)):

            if results[p]==[] and len(results)==1:
                self.CalcPlot.AxSet(self.ax,chunk[1][1],chunk[1][2])
                continue

            if results[p]==[]:
                self.CalcPlot.AxSet(self.ax[p],chunk[1][1],chunk[1][2])


            if len(results)==1:
                d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                self.CalcPlot.AxSet(self.ax,chunk[1][1],chunk[1][2])
                self.ax.spines['bottom'].set_visible(False)
                self.CalcPlot.plotChunk(d,self.ax,chunk[1][1],chunk[1][2])

                #self.CalcPlot.PlotNucChunk(d,self.ax,chunk[1][1],chunk[1][2],flag=self.flag)
                continue

            if p==0:
                d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                self.CalcPlot.AxSet(self.ax[p],chunk[1][1],chunk[1][2],chunk=True)
                self.ax[p].spines['bottom'].set_visible(False)
                self.CalcPlot.plotChunk(d,self.ax[p],chunk[1][1],chunk[1][2])
            else:
                if results[p-1]==[]:
                    d=pd.DataFrame(chunk[0],columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
                    self.ax[p].spines['bottom'].set_visible(False)
                    self.CalcPlot.AxSet(self.ax[p],chunk[1][1],chunk[1][2],chunk=True)
                    self.CalcPlot.plotChunk(d,self.ax[p],chunk[1][1],chunk[1][2])
                else:
                    d=self.CalcMapping.PlotreadsDF(results[p-1],results[p],multi[p-1][2])
                    self.ax[p].spines['bottom'].set_visible(False)
                    self.CalcPlot.AxSet(self.ax[p],chunk[1][1],chunk[1][2],chunk=True)
                    self.CalcPlot.plotChunk(d,self.ax[p],chunk[1][1],chunk[1][2])



import pysam
import pandas as pd
import matplotlib.pyplot as plt
from tqdm import tqdm
from multiprocessing import Pool

class CalcMapping():
    def __init__(self,mapping,chrom,start,end,threads=1,coverage=200,flag='None',chunksize=1000):
        self.mapping=mapping
        self.chrom=chrom
        self.start=start-1
        self.end=end
        self.maxHeight=coverage
        self.Fontsize=3
        self.threads=threads
        self.flag=flag
        self.chunksize=chunksize

    def plotList(self,chrom,start,end,forwardOnly,reverseOnly):
        df=pd.DataFrame(0,index=range(0,self.maxHeight+50),columns=range(start,end+1))
        plotList=[]
        with pysam.AlignmentFile(self.mapping) as s:
            for record in tqdm(s.fetch(str(chrom),start,end,until_eof=True)):
                #print(df)
                #print(list(df))


                if not record.is_unmapped:
                    if record.is_reverse:
                        if forwardOnly:
                            continue
                    if not record.is_reverse:
                        if reverseOnly:
                            continue

                #    if record.reference_start>=start and record.reference_end<end:
                #        if df.loc[self.maxHeight,start:end].sum()>=((self.end-self.start)*0.9):
                #            print('hardbreak')
                #            break

                #        if df.loc[self.maxHeight+50-1,record.reference_start:record.reference_end].sum()>=((record.reference_end-record.reference_start)*0.9):
                #            continue
                    endPos=record.reference_end
                    if record.reference_end>=end:
                        endPos=end
                    startPos=record.reference_start
                    if record.reference_start<=start:
                        startPos=start
                    if sum(df[range(startPos,endPos+1)].sum(axis=1)) == 0:
                        for _ in range(startPos,endPos+1):
                            if _ >=(end):
                                continue
                            df.at[0,_]=1
                        if record.is_reverse:

                            plotList.append((0,record.reference_start,record.reference_end,'r',
                                             record.qname+'_R',
                                             record.query_alignment_sequence,
                                             record.cigarstring,
                                             record.mate_is_unmapped))
                        if not record.is_reverse:

                            plotList.append((0,record.reference_start,record.reference_end,'f',
                                             record.qname+'_F',
                                             record.query_alignment_sequence,
                                             record.cigarstring,
                                             record.mate_is_unmapped))


                    else:

                        for p,v in enumerate(list(df[range(startPos,endPos+1)].sum(axis=1))):
                            if v ==0:
                                for _ in range(startPos,endPos+1):
                                    df.at[p,_]=1

                                if record.is_reverse:

                                    plotList.append((p,record.reference_start,record.reference_end,'r',
                                                     record.qname+'_R',
                                                     record.query_alignment_sequence,
                                                     record.cigarstring,
                                                     record.mate_is_unmapped))
                                if not record.is_reverse:

                                    plotList.append((p,record.reference_start,record.reference_end,'f',
                                                     record.qname+'_F',
                                                     record.query_alignment_sequence,
                                                     record.cigarstring,
                                                     record.mate_is_unmapped))
                                break
        return plotList

    def plotListRVRS(self,chrom,start,end):
        df=pd.DataFrame(0,index=range(0,self.maxHeight+50),columns=range(start,end+1))
        plotList=[]
        with pysam.AlignmentFile(self.mapping) as s:
            for record in tqdm(s.fetch(str(chrom),start,end,until_eof=True)):
                if not record.is_unmapped and record.is_reverse:
                    #print('s')
                    #print(record.reference_start)
                    #record.reference_start=record.reference_start+1
                    #record.reference_end=record.reference_end+1
                    #print(record.reference_start)
                    #print('e')
                    for _ in range(record.reference_start,record.reference_start+self.chunk_size+100):
                        if _ not in list(df):
                            df[_]=0
                    if sum(df[range(record.reference_start,record.reference_end)].sum(axis=1)) == 0:
                        for _ in range(record.reference_start,record.reference_end):

                            df.at[0,_]=1

                        plotList.append((0,record.reference_start,record.reference_end,'r',
                                             record.qname+'_R',
                                             record.query_alignment_sequence,
                                             record.cigarstring,
                                             record.mate_is_unmapped))


                    else:
                        for p,v in enumerate(list(df[range(record.reference_start,record.reference_end)].sum(axis=1))):
                            if v ==0:
                                for _ in range(record.reference_start,record.reference_end):
                                    df.at[p,_]=1
                                plotList.append((p,record.reference_start,record.reference_end,'r',
                                                     record.qname+'_R',
                                                     record.query_alignment_sequence,
                                                     record.cigarstring,
                                                     record.mate_is_unmapped))
                                break
        return plotList

    def plotListFRWRD(self,chrom,start,end):
        df=pd.DataFrame(0,index=range(0,self.maxHeight+50),columns=range(start,end+1))
        plotList=[]
        with pysam.AlignmentFile(self.mapping) as s:
            for record in tqdm(s.fetch(str(chrom),start,end,until_eof=True)):
                if not record.is_unmapped and not record.is_reverse:
                    for _ in range(record.reference_start,record.reference_end+1):
                        if _ not in list(df):
                            df[_]=0
                    if sum(df[range(record.reference_start,record.reference_end+1)].sum(axis=1)) == 0:
                        for _ in range(record.reference_start,record.reference_end+1):
                            df.at[0,_]=1

                        plotList.append((0,record.reference_start,record.reference_end,'f',
                                             record.qname+'_F',
                                             record.query_alignment_sequence,
                                             record.cigarstring,
                                             record.mate_is_unmapped))


                    else:
                        for p,v in enumerate(list(df[range(record.reference_start,record.reference_end+1)].sum(axis=1))):
                            if v ==0:
                                for _ in range(record.reference_start,record.reference_end+1):
                                    df.at[p,_]=1
                                plotList.append((p,record.reference_start,record.reference_end,'f',
                                                     record.qname+'_F',
                                                     record.query_alignment_sequence,
                                                     record.cigarstring,
                                                     record.mate_is_unmapped))
                                break

        return plotList



    def PlotreadsDF(self,lbefore,l,end_prevChunk):
        before=pd.DataFrame(lbefore,columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
        before=before[before['end']>end_prevChunk]
        recent=pd.DataFrame(l,columns=['y','start','end','direction','name','qSeq','cigar','mateMap'])
        for overlappRead in set(before['name']).intersection(set(recent['name'])):
            ybefore=before[before['name']==overlappRead]['y'].values[0]
            beforeReads=before[before['y']==ybefore]['name']
            yrecent=recent[recent['name']==overlappRead]['y'].values[0]
            recentReads=recent[recent['y']==yrecent]['name']
            recenUpdateReads=recent[recent['y']==ybefore]['name']

            for n in recenUpdateReads:
                i=recent[recent['name']==n].index.values[0]
                recent.at[i,'y']=yrecent

            for n in recentReads:
                i=recent[recent['name']==n].index.values[0]
                recent.at[i,'y']=ybefore
        return recent



import pysam
import pandas as pd
import matplotlib.pyplot as plt
from tqdm import tqdm
from multiprocessing import Pool
import configparser
import os
import sys
import matplotlib.patches as patches
class CalcPlot():
    def __init__(self,mapping,chrom,start,end,threads=1,coverage=200,flag='None',chunksize=1000, fasta=None,style='classic',outlineoff=False):
        self.mapping=mapping
        self.chrom=chrom
        self.start=start
        self.end=end
        self.maxHeight=coverage
        self.Fontsize=3
        self.threads=threads
        self.flag=flag
        self.chunksize=chunksize
        self.fasta = fasta
        self.outlineoff=outlineoff
        config = configparser.ConfigParser()
        if style!=None:
            if os.path.isfile(style):
                config.read(style)
            else:
                print('{} is not a existing file'.format(style))
                config.read(os.path.join(os.path.dirname(__file__),'classic.ini'))

        else:
            config.read(os.path.join(os.path.dirname(__file__),'classic.ini'))

        colorDict={}
        for a in config['nucleotide color']:
            colorDict[a.upper()]=config['nucleotide color'][a]
            colorDict[a.lower()]=config['nucleotide color'][a]
        for a in config['special chars']:
            colorDict[a]=config['special chars'][a]
        for a in config['MatplotStyle']:
            colorDict[a]=config['MatplotStyle'][a]
        self.colorDict=colorDict
        plt.style.use(self.colorDict['pltstyle'])


        #self.colorDict={'A':'red','C':'blue','T':'green','G':'yellow','-':'white','N':'pink','a':'red','c':'blue','t':'green','g':'yellow','-':'white','n':'pink','|':'black'}
    def startPlot(self,cols,direction,schematic):

        self.cols=cols
        self.innerLW=0
        self.LW=0.2
        if not direction and not schematic:
            if cols==1:
                if self.end-self.start <=100:
                    x=0.07*(self.end-self.start)
                    y=0.12*self.maxHeight
                    fig,ax=plt.subplots(1,cols,figsize=(x,y))
                    self.Fontsize=6
                    self.markersize=0.5
                else:
                    x=0.035*(self.end-self.start)
                    y=0.06*self.maxHeight
                    self.markersize=0.25

                    fig,ax=plt.subplots(1,cols,figsize=(x,y))
            if cols >1:
                x=0.035*5000
                y=0.06*self.maxHeight
                fig,ax=plt.subplots(1,cols,figsize=(x*cols,6))
        if direction and not schematic:
            if cols==1:
                if self.end-self.start <=100:
                    x=0.07*(self.end-self.start)
                    y=0.12*self.maxHeight*2
                    self.markersize=0.5

                    fig,ax=plt.subplots(2,cols,figsize=(x,y))
                    self.Fontsize=6
                else:
                    x=0.035*(self.end-self.start)
                    y=0.06*self.maxHeight*2
                    self.markersize=0.25
                    fig,ax=plt.subplots(2,cols,figsize=(x,y))
            if cols >1:
                x=0.035*5000
                y=0.06*self.maxHeight*2
                fig,ax=plt.subplots(2,cols,figsize=(x*cols,6))


        if not direction and schematic:
            if self.cols >= 5:
                y=(0.12*self.maxHeight)/5
                x=12
                self.innerLW=0
                self.LW=0.2
            else:
                self.innerLW=0.4/self.cols
                self.LW=3/self.cols
                x=12
                y=(0.12*self.maxHeight)/self.cols
            fig,ax=plt.subplots(1,self.cols,figsize=(x,y))
        if  direction and schematic:
            if self.cols >= 5:
                y=((0.12*self.maxHeight)/5)*2
                x=12
                self.innerLW=0
                self.LW=0.2
            else:
                self.innerLW=0.4/self.cols
                self.LW=3/self.cols
                x=12
                y=((0.12*self.maxHeight)/self.cols)*2
            fig,ax=plt.subplots(2,self.cols,figsize=(x,y))

        plt.suptitle('chromosome {} from {} to {}'.format(str(self.chrom),str(self.start),str(self.end)))
        self.ax=ax
        self.fig=fig
        plt.subplots_adjust(wspace=0,hspace=0.05, )
        return self.fig,self.ax


    def plotChunk(self,df,ax,start,end,direction=None):
        df['y']=df['y']+1

        for y,s,e,d in zip(df['y'],df['start'],df['end'],df['direction']):
            e=e
            s=s
            if y>(self.maxHeight):
                ax.plot((s,e),(self.maxHeight+1,self.maxHeight+1),color='red',alpha=0.1)
                continue
            if d=='r':
                ax.plot((s,e),(y,y),color='grey',linewidth=self.LW)
                ax.plot((s+1,e-1),(y,y),linewidth=self.innerLW,color='white')

            else:
                ax.plot((s,e),(y,y),color='black',linewidth=self.LW)
                ax.plot((s+1,e-1),(y,y),linewidth=self.innerLW,color='white')



    def AxSet(self,ax,start,end,chunk=False,direction='all',vcf=False):
        ylabel=direction
        if not chunk:
            ax.set(xlim=(start,end),ylim=(0,self.maxHeight+2),ylabel=ylabel)
            if vcf:
                ax.set_xticks([start,start+(end-start)/2,end],)
                ax.set_xticklabels([str("{:,}".format(start)),'vcf position',str("{:,}".format(end))], rotation=40, ha='right')
            else:
                ax.set_xticklabels([str("{:,}".format(start)),str("{:,}".format(end))], rotation=40, ha='right')
                ax.set_xticks([start,end],)

        else:
            ax.get_yaxis().set_visible(False)
            ax.set(xlim=(start,start+self.chunksize),ylim=(0,self.maxHeight+2))
            if start==self.start:
                ax.set(ylabel=ylabel)
                ax.get_yaxis().set_visible(True)
                ax.set_xticks([start])
                ax.set_xticklabels([str("{:,}".format(start))], rotation=40, ha='right',fontsize=6)
            if end==self.end:
                ax.set_xticks([start,start+self.chunksize])
                ax.set_xticklabels(['',str((start+self.chunksize)/1_000_000)], rotation=40, ha='right',fontsize=6)
            if  start!=self.start and end!=self.end:
                ax.set_xticks([start])
                ax.set_xticklabels([''], rotation=40, ha='right',fontsize=6)


        ax.spines['right'].set_visible(False)
        ax.spines['left'].set_visible(False)

    def CigChunker(self,cig):
        cigL=[]
        def parseCig(cig):
            for p,l in enumerate(cig):
                if l.isalpha():
                    liste=([cig[p]]*int(cig[:p]))
                    cig=cig[p+1:]
                    return cig,liste
        while cig:
            cigL=cigL+parseCig(cig)[1]
            cig=parseCig(cig)[0]
        return cigL



    def PlotFasta(self,ax):
        if self.fasta=='None':
            return
        with pysam.FastaFile(self.fasta) as fa:
            for _,n in enumerate(fa.fetch(self.chrom,self.start-1,self.end)):
                ax.text(self.start+_,0,n,fontsize=self.Fontsize,
                        color=self.colorDict[n],alpha=1,family='monospace',ha='center',va='center',
                        bbox=dict(alpha=1,boxstyle='square,pad=0', fc=self.colorDict['fasta background'], ec='none'))

    def listConsec(self,l):
        retL=[]
        c=0
        for e,x in enumerate(l):
            if x+1 not in l:
                retL.append(l[c:e+1])
                c=e+1
        return(retL)
    def PlotNucChunk(self,df,ax,start,end,flag='None'):
        plotC=set()
        #if self.readoutline:
    #        self.plotChunk(df,ax,self.start,self.end)
        df['y']=df['y']+1
        for y,s,e,d,qS,cig,mate in zip(df['y'],df['start'],df['end'],df['direction'],df['qSeq'],df['cigar'],df['mateMap']):
            if self.fasta != 'None':
                with pysam.FastaFile(self.fasta) as fa:
                    fastaChunk=str(fa.fetch(self.chrom,s,e)).upper()
            e=e+1
            s=s+1
            if y>self.maxHeight:
                ax.plot((s,e),(self.maxHeight+1,self.maxHeight+1),color=self.colorDict['max coverage'],alpha=0.1)
                continue

            chunk_cigarstring=self.CigChunker(cig)
            query_alignment_sequence=qS
            chunkL=len(chunk_cigarstring)
            chunk_cigarstringS=[x for x in chunk_cigarstring if x =='S' or x =='H']
            chunk_cigarstring=[x for x in chunk_cigarstring if x !='S' and x !='H']

            ipos=[x for x,y in enumerate(chunk_cigarstring) if y=='I']
            ipos=self.listConsec(ipos)
            chunk_cigarstring=[x for x in chunk_cigarstring if x !='I']
            iposCounter=0

            for i in ipos:
                query_alignment_sequence="".join([x for _,x in enumerate(query_alignment_sequence) if _+iposCounter not in i])
                iposCounter=iposCounter+len(i)

            for p,_ in enumerate(chunk_cigarstring):
                if _=='D' or _=='N':
                    qs1=query_alignment_sequence[:p]
                    qs2=query_alignment_sequence[p:]
                    query_alignment_sequence=qs1+'-'+qs2


            softClipp=len(chunk_cigarstringS)/chunkL
            alpha=1
            if flag=='None':
                alpha=1
            if flag=='MateUnmapped':
                if mate:
                    alpha=0.3
            if flag=='SoftClipped':
                if softClipp>=0.05:
                    alpha=0.3
            if flag=='MateUnmappedSoftClipped':
                if softClipp>=0.1 or mate:
                    alpha=0.3
            fastapos=0
            drawChunk=[]
            if not self.outlineoff:
                ax.plot((s,s+len(query_alignment_sequence)-1),(y,y),color='black',linewidth=2.6,zorder=0)
                ax.plot((s+0.1,s+len(query_alignment_sequence)-1-0.1),(y,y),color='darkgrey',linewidth=2.2,alpha=0.5,zorder=0)

            fontDict={'A':r'$\mathtt{A}$','C':r'$\mathtt{C}$','G':r'$\mathtt{G}$','T':r'$\mathtt{T}$','-':r'$\mathtt{-}$'}
            xs=[x+s for x in range(0,len(chunk_cigarstring))]
            ys=len(chunk_cigarstring)*[y]
            if self.fasta=='None':
                for _ in ['A','C','T','G','-']:
                    xp=[x for x,nuc in zip(xs,query_alignment_sequence) if nuc == _]
                    yp=[y for y,nuc in zip(ys,query_alignment_sequence) if nuc == _]
                    ax.scatter(xp,yp,marker=fontDict[_],lw=0,color=self.colorDict[_],s=4,alpha=alpha)
                continue

            if self.fasta != 'None':
                ax.scatter([x+s for x in range(0,len(chunk_cigarstring))],len(chunk_cigarstring)*[y], marker='o',s=self.markersize,color=self.colorDict['dot'],alpha=alpha)

            for p,alignPos in enumerate(chunk_cigarstring):
                x=s+p
                if x>self.end:
                    continue
                if x<self.start:
                    fastapos=fastapos+1
                    continue
                if alignPos=='S':
                    continue
                if alignPos=='N':
                    ax.text(x,y,query_alignment_sequence[p], fontsize=self.Fontsize,ha='center',va='center',family='monospace',
                            color=self.colorDict['-'],bbox=dict(alpha=alpha,boxstyle='square,pad=0', fc=self.colorDict['gap background'], ec='darkgrey',linewidth=0.00001))
                    fastapos=fastapos+1
                    continue
                if alignPos=='M':
                    if self.fasta != 'None' and fastaChunk[fastapos]==query_alignment_sequence[p]:
                        fastapos=fastapos+1
                        continue
                    else:
                        if self.fasta != 'None':
                            ax.text(x,y,query_alignment_sequence[p],fontsize=self.Fontsize,
                                color=self.colorDict['nuc missmatch font'],alpha=alpha,family='monospace',ha='center',va='center_baseline',zorder=2
                                )
                                #bbox=dict(alpha=alpha,boxstyle='square,pad=0.1' ,fc=self.colorDict[query_alignment_sequence[p]], ec='none')
                            rect = patches.Rectangle((x-0.45, y-0.5),
                            1,1,edgecolor='none',facecolor=self.colorDict[query_alignment_sequence[p]],zorder=1)
# add rectangle to plot
                            ax.add_patch(rect)
                            fastapos=fastapos+1
                            continue

                if alignPos=='D':
                    ax.text(x,y,query_alignment_sequence[p], fontsize=self.Fontsize,ha='center',va='center',family='monospace',
                            color=self.colorDict['-'],bbox=dict(alpha=alpha,boxstyle='square,pad=0', fc=self.colorDict['gap background'], ec='darkgrey',linewidth=0.00001))
                    fastapos=fastapos+1
                    continue

            if ipos!=[]:
                for i in ipos:
                    minimum=min(i)
                    x=s+minimum
                    for ii in i:
                        ax.plot((x,x),(y-0.5,y+0.5),linewidth=0.5, color=self.colorDict['insertion'])
                        x=x+0.05
